﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.MultiTouch.Gestures;

namespace Infragistics.MultiTouch.Gestures.Controls
{
    public class GestureEventHandlerMap : Dictionary<Gesture, EventHandler>
	{
		#region Constructor
		/// <summary>
		/// Initializes a new instance of the <see cref="GestureEventHandlerMap"/> class.
		/// </summary>
		internal GestureEventHandlerMap()
        {
        }
		#endregion Constructor

		#region Public Methods
		/// <summary>
        /// Registers an event handler with the specifed gesture.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="handler"></param>
        public new void Add(Gesture g, EventHandler handler)
        {
            if (ContainsKey(g))
            {
                this[g] += handler;
            }
            else
            {
                base.Add(g, handler);
            }
        }

        /// <summary>
        /// Unregisters a specific event handler for the given gesture.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="handler"></param>
        public void Remove(Gesture g, EventHandler handler)
        {
            if (ContainsKey(g))
            {
                this[g] -= handler;

                if (this[g] == null || this[g].GetInvocationList().Length == 0)
                {
                    Remove(g);
                }
            }
        }

		/// <summary>
		/// Removes all handlers from the specified gesture.
		/// </summary>
		/// <param name="g">The g.</param>
        public new void Remove(Gesture g)
        {
            if (ContainsKey(g))
            {
                Remove(g);
            }
        }
		#endregion Public Methods

		#region Internal Methods
		/// <summary>
        /// Returns a value indicating if one or more handlers have been registered for the 
        /// specified gesture.
        /// </summary>
        /// <param name="g"></param>
        /// <returns></returns>
        internal bool CanInvoke(Gesture g)
        {
            return (ContainsKey(g));
        }

        /// <summary>
        /// Invokes all registered handlers for the specified gesture. Note:
        /// an exception is thrown if no registered handlers are found for the gesture.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal void Invoke(Gesture g, object sender, EventArgs e)
        {
            InvokeInternal(g, sender, e, true);
        }

        /// <summary>
        /// Invokes all registered handlers for the specified gesture. Returns
        /// a value indicating the handlers could be invoked.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        internal bool SafeInvoke(Gesture g, object sender, EventArgs e)
        {
            return InvokeInternal(g, sender, e, false);
        }
		#endregion Internal Methods

		#region Private Methods
		/// <summary>
        /// Invokes the handlers for the specified gesture.
        /// </summary>
        /// <param name="g"></param>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <param name="raiseExceptionIfNotFound">Indicates if an exception should be raised if the gesture is not mapped to any handlers.</param>
        /// <returns></returns>
        private bool InvokeInternal(Gesture g, object sender, EventArgs e, bool raiseExceptionIfNotFound)
        {
            bool invoked = false;

            if (ContainsKey(g))
            {
                this[g].Invoke(sender, e);

                invoked = true;
            }
            else if (raiseExceptionIfNotFound)
            {
                throw new InvalidOperationException("The specified gesture was not found in the event map collection.");
            }

            return invoked;
		}
		#endregion Private Methods
	}
}
