﻿using System;
using System.Windows;
using System.Windows.Shapes;
using System.Windows.Input;
using System.Windows.Media;

namespace Infragistics.MultiTouch.Gestures.Controls
{
    public class GestureProcessor
    {
        #region Private Member Variables
        #region Private Event Handlers
        private RoutedEventHandler          _gestureStartedEvent;
        private GestureRoutedEventHandler   _gestureCompletedEvent;
        private GestureRoutedEventHandler   _gestureDirectionChangedEvent;
        #endregion Private Event Handlers

        private IGestureProvider            _gestureProvider;

        private FrameworkElement            _owner;                   
        #endregion Private Member Variables

        #region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="GestureProcessor"/> class.
		/// </summary>
		/// <param name="owner">The owner.</param>
        public GestureProcessor(FrameworkElement owner)
        {
            _owner = owner;

            _gestureProvider = new GestureProvider();
            _gestureProvider.GestureDirectionChanged += new GestureEventHandler(gestureProvider_GestureDirectionChanged);

			GestureHandlers = new GestureEventHandlerMap();

            _owner.MouseEnter 			+= GestureProcessor_MouseEnter;
            _owner.MouseLeave 			+= GestureProcessor_MouseLeave;
            _owner.MouseMove 			+= GestureProcessor_MouseMove;
            _owner.MouseLeftButtonDown 	+= GestureProcessor_MouseLeftButtonDown;
            _owner.MouseLeftButtonUp 	+= GestureProcessor_MouseLeftButtonUp;
        }
        #endregion Constructors

        #region Event Handlers
        /// <summary>
        /// The event raised when a gesture is started.
        /// </summary>
        public event RoutedEventHandler GestureStarted
        {
            add { _gestureStartedEvent += value; }
            remove { _gestureStartedEvent -= value; }
        }

        /// <summary>
        /// The event raised when a gesture is completed.
        /// </summary>
        public event GestureRoutedEventHandler GestureCompleted
        {
            add { _gestureCompletedEvent += value; }
            remove { _gestureCompletedEvent -= value; }
        }

        /// <summary>
        /// The event raised when a gesture direction change is reported.
        /// </summary>
        public event GestureRoutedEventHandler GestureDirectionChanged
        {
            add { _gestureDirectionChangedEvent += value; }
            remove { _gestureDirectionChangedEvent -= value; }
        }

        /// <summary>
        /// Gets the control's event handler delegates list.
        /// </summary>
        public GestureEventHandlerMap GestureHandlers
        {
            get; private set;
        }
        #endregion Event Handlers

		#region Event Handlers
		/// <summary>
		/// Handles the MouseMove event of the GestureProcessor control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Input.MouseEventArgs"/> instance containing the event data.</param>
		void GestureProcessor_MouseMove(object sender, MouseEventArgs e)
        {
            if (!_gestureProvider.IsCapturing)
            {
                return;
            }

            Point location = e.GetPosition(e.OriginalSource as FrameworkElement);

            _gestureProvider.CapturePoint(location);
        }

		/// <summary>
		/// Handles the MouseLeave event of the GestureProcessor control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Input.MouseEventArgs"/> instance containing the event data.</param>
        void GestureProcessor_MouseLeave(object sender, MouseEventArgs e)
        {
            EndCapture();
        }

		/// <summary>
		/// Handles the MouseEnter event of the GestureProcessor control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Input.MouseEventArgs"/> instance containing the event data.</param>
        void GestureProcessor_MouseEnter(object sender, MouseEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine(e.StylusDevice.DeviceType.ToString());
        }

		/// <summary>
		/// Handles the MouseLeftButtonUp event of the GestureProcessor control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Input.MouseButtonEventArgs"/> instance containing the event data.</param>
        void GestureProcessor_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            EndCapture();
        }

		/// <summary>
		/// Handles the MouseLeftButtonDown event of the GestureProcessor control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.Input.MouseButtonEventArgs"/> instance containing the event data.</param>
        void GestureProcessor_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            PathGeometry g = new PathGeometry();

            Point location = e.GetPosition(e.OriginalSource as FrameworkElement);
            _gestureProvider.StartCapture(location);

            RaiseOnGestureStarted(new RoutedEventArgs());

            g.Figures = new PathFigureCollection();
            g.Figures.Add(new PathFigure() { StartPoint = location, Segments = new PathSegmentCollection(), IsFilled = false, IsClosed = false });
        }
		#endregion Event Handlers

		#region Events
		/// <summary>
		/// Raises the on gesture started.
		/// </summary>
		/// <param name="args">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
		protected virtual void RaiseOnGestureStarted(RoutedEventArgs args)
        {
            if (_gestureStartedEvent == null) return;

            _owner.Dispatcher.BeginInvoke(_gestureStartedEvent, new object[] { this, args });
        }

		/// <summary>
		/// Raises the on gesture completed.
		/// </summary>
		/// <param name="args">The <see cref="Infragistics.MultiTouch.Gestures.Controls.GestureRoutedEventArgs"/> instance containing the event data.</param>
        protected virtual void RaiseOnGestureCompleted(GestureRoutedEventArgs args)
        {
            if (_gestureCompletedEvent == null) return;

            _owner.Dispatcher.BeginInvoke(_gestureCompletedEvent, new object[] { this, args });
        }

		/// <summary>
		/// Raises the on gesture direction changed.
		/// </summary>
		/// <param name="args">The <see cref="Infragistics.MultiTouch.Gestures.Controls.GestureRoutedEventArgs"/> instance containing the event data.</param>
        protected virtual void RaiseOnGestureDirectionChanged(GestureRoutedEventArgs args)
        {
            if (_gestureDirectionChangedEvent == null) return;

            _owner.Dispatcher.BeginInvoke(_gestureDirectionChangedEvent, new object[] { this, args });
        }
		#endregion Events

		#region Private Methods
		/// <summary>
        /// Retrieves the completed gesture, and raises the <see cref="GestureCompleted"/> event.
        /// </summary>
        private void EndCapture()
        {
            if (!_gestureProvider.IsCapturing)
            {
                return;
            }

            Gesture g = _gestureProvider.StopCapture();

            RaiseOnGestureCompleted(new GestureRoutedEventArgs(this, g));

            GestureHandlers.SafeInvoke(g, this, EventArgs.Empty);
        }

        /// <summary>
        /// Invoked when a gesture direction change is reported.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void gestureProvider_GestureDirectionChanged(object sender, GestureEventArgs e)
        {
            GestureRoutedEventArgs args = new GestureRoutedEventArgs(this, e.Gesture);

            RaiseOnGestureDirectionChanged(args);
		}
		#endregion Private Methods
	}
}
