﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Shapes;

namespace Infragistics.MultiTouch.Gestures
{
    public class GestureProvider : IGestureProvider
    {
        #region Public Event
        public event GestureEventHandler GestureDirectionChanged;
        public event GestureEventHandler GestureCompleted;
        #endregion Public Event

        #region Private Constants
        private const double MIN_TRAVEL         = 7.5;
        private const double SMALL_ANGLE_CHANGE = 7.5;
        #endregion Private Constants

        #region Private Member Variables
        private Gesture     _currentGesture;
        private Point?      _pointOffset;
        private Direction   _currentDirection;
        private double      _currentAngle;
        #endregion Private Member Variables

        #region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="GestureProvider"/> class.
		/// </summary>
        public GestureProvider()
        {
            _currentDirection = Direction.NotSet;
        }
        #endregion Constructors

        #region Public Properties
		/// <summary>
		/// Gets a value indicating if the provider is currently in point capture mode.
		/// </summary>
		/// <value></value>
        public bool IsCapturing
        {
            get { return (_pointOffset != null); }
        }
        #endregion Public Properties

        #region Public Methods
		/// <summary>
		/// Starts a new capture.
		/// </summary>
		/// <param name="startPoint"></param>
        public void StartCapture(Point startPoint)
        {
            _pointOffset 		= startPoint;
            _currentGesture 	= new Gesture();
            _currentDirection 	= Direction.NotSet;
        }

		/// <summary>
		/// Ends the current capture and returns the completed gesture.
		/// </summary>
		/// <returns></returns>
        public Gesture StopCapture()
        {
			if (!IsCapturing)
			{
				return null;
			}

            RaiseOnGestureCompleted(new GestureEventArgs(_currentGesture));

            Gesture g = _currentGesture;

            _currentGesture 	= null;
            _currentDirection 	= Direction.NotSet;
            _pointOffset 		= null;

            return g;
        }

		/// <summary>
		/// Captures a point.
		/// </summary>
		/// <param name="point"></param>
        public void CapturePoint(Point point)
        {
            UpdateCurrentDirection(point);
        }
        #endregion Public Methods

        #region Protected Methods
		/// <summary>
		/// Raises the <see cref="E:GestureCompleted"/> event.
		/// </summary>
		/// <param name="args">The <see cref="Infragistics.MultiTouch.Gestures.GestureEventArgs"/> instance containing the event data.</param>
        protected virtual void RaiseOnGestureCompleted(GestureEventArgs args)
        {
			if (GestureCompleted == null) return;

			GestureCompleted(this, args);
        }

		/// <summary>
		/// Raises the on gesture direction changed.
		/// </summary>
		/// <param name="args">The <see cref="Infragistics.MultiTouch.Gestures.GestureEventArgs"/> instance containing the event data.</param>
        protected virtual void RaiseOnGestureDirectionChanged(GestureEventArgs args)
        {
			if (GestureDirectionChanged == null) return;

			GestureDirectionChanged(this, args);
        }
        #endregion Protected Methods

        #region Private Methods
		/// <summary>
		/// Updates the current direction.
		/// </summary>
		/// <param name="point">The point.</param>
        private void UpdateCurrentDirection(Point point)
        {
            //calculate direction and angle offset 
            double alpha;
            Direction dir = CalculateDirection(point, out alpha);

            double diff = (_currentAngle > alpha) ? _currentAngle - alpha : alpha - _currentAngle;

            //if the angle is changing, update the master offset and current angle
            if (diff >= SMALL_ANGLE_CHANGE)
            {
                _pointOffset   = point;
                _currentAngle  = alpha;
            }

            //if direction has changed, update the current direction
            if (dir != _currentDirection && dir != Direction.NotSet)
            {
                _currentDirection = dir;
                _currentGesture.Add(_currentDirection);

                //raise the direction changed event
                RaiseOnGestureDirectionChanged(new GestureEventArgs(_currentGesture));
            }
        }

		/// <summary>
		/// Calculates the direction.
		/// </summary>
		/// <param name="location">The location.</param>
		/// <param name="alpha">The alpha.</param>
		/// <returns></returns>
        private Direction CalculateDirection(Point location, out double alpha)
        {
            Direction dir = Direction.NotSet;

            // Clear out our alpha variable
            alpha = 0;

            //tranform the end point to window space 
            Point to = new Point(location.X - _pointOffset.Value.X, location.Y - _pointOffset.Value.Y);

            //no movement=no direction!
            if (to.X == 0 && to.Y == 0)
            {
                return Direction.NotSet;
            }

            double magnitude = Math.Sqrt(to.X * to.X + to.Y * to.Y);

            //filter out epsilon values
            if (magnitude < MIN_TRAVEL)
            {
                return _currentDirection;
            }

            //calcuate angle of movement
            double angle = Math.Asin(Math.Abs(to.X) / magnitude);
            
            alpha = Math.Round((angle * 180 / Math.PI), 2);

            //adjust for orientation
            if (to.X > 0)
            {
                if (to.Y < 0)
                {
                    alpha = 180 - alpha;
                }
            }
            else
            {
                if (to.Y > 0)
                {
                    alpha = -alpha;
                }
                else
                {
                    alpha = alpha + 180;
                }
            }

            //get the direction
            if (alpha >= 225 && alpha < 315)
            {
                dir = Direction.Left;
            }
            else if ((alpha >= 0 && alpha < 45) || alpha >= 315)
            {
                dir = Direction.Down;
            }
            else if (alpha >= 45 && alpha < 135)
            {
                dir = Direction.Right;
            }
            else if (alpha >= 135 && alpha < 225)
            {
                dir = Direction.Up;
            }

            return dir;
        }
        #endregion Private Methods
    }
}
