﻿using System;
using System.Windows;
using System.Collections.Generic;
using System.Threading;
using System.Linq;
using Infragistics.Silverlight.Applications.Stocks.Models;


namespace Infragistics.Silverlight.Applications.Stocks.Services
{
    public class LocalStockDataService : IApplicationService, IStockDataProvider
    {
        private StockDataCacheService _cache;

        /// <summary>
        /// Called by an application in order to initialize the application extension service.
        /// </summary>
        /// <param name="context">Provides information about the application state.</param>
        public void StartService(ApplicationServiceContext context)
        {
            _cache = ServiceLocator.GetInstance<StockDataCacheService>();

            InitializeData();
        }

        /// <summary>
        /// Called by an application in order to stop the application extension service.
        /// </summary>
        public void StopService()
        {
        }

        /// <summary>
        /// Refreshes the details.
        /// </summary>
        /// <param name="symbols">The symbols.</param>
        /// <param name="callback">The callback.</param>
        public void RefreshDetails(IList<string> symbols, Action<IDictionary<string, Models.StockTickerDetails>, Exception, bool> callback)
        {
            string s = symbols.Aggregate((x, y) => string.Format("{0}+{1}", x, y));

            SynchronizationContext.Current.Post(x => callback((IDictionary<string, StockTickerDetails>)x, null, true), GetStockDetails(symbols));
        }

        /// <summary>
        /// Refreshes the data.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <param name="startDate">The start date.</param>
        /// <param name="callback">The callback.</param>
        public void RefreshData(string symbol, DateTime startDate, Action<string, IEnumerable<Models.StockTickerData>, Exception, bool> callback)
        {
            RefreshData(symbol, startDate, DateTime.Now.AddDays(-1.0), callback);
        }

        public void RefreshData(string symbol, DateTime startDate, DateTime endDate, Action<string, IEnumerable<Models.StockTickerData>, Exception, bool> callback)
        {
            SynchronizationContext.Current.Post(x => callback(symbol, (IEnumerable<StockTickerData>)x, null, true), GetStockData(symbol));
        }

        /// <summary>
        /// Initializes the data.
        /// </summary>
        private void InitializeData()
        {
            _cache.UpdateCache("GOOG", GenerateStockTickerData(360, 100, 550));
            _cache.UpdateCache("MSFT", GenerateStockTickerData(360, 10, 45));
            _cache.UpdateCache("AAPL", GenerateStockTickerData(360, 1, 250));
            _cache.UpdateCache("IBM", GenerateStockTickerData(360, 9, 35));
        }

        /// <summary>
        /// Gets the stock details.
        /// </summary>
        /// <param name="symbols">The symbols.</param>
        /// <returns></returns>
        private IDictionary<string, StockTickerDetails> GetStockDetails(IList<string> symbols)
        {
            var result = new Dictionary<string, StockTickerDetails>();

            foreach (var symbol in symbols)
            {
                var data = GenerateStockTickerData(1, 1, 500).First();

                result.Add(symbol, new StockTickerDetails
                                        {
                                            Symbol = symbol,
                                            Name = symbol,
                                            Bid = data.Low, //.ToString(),
                                            Ask = (data.Low + 1), //.ToString(),
                                            PercentChange = string.Format("5.5% - {0}", data.Low)
                                        });

            }

            return result;
        }

        /// <summary>
        /// Gets the stock data.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <returns></returns>
        private IEnumerable<StockTickerData> GetStockData(string symbol)
        {
            IEnumerable<StockTickerData> result;

            if (!_cache.IsSymbolInCache(symbol))
            {
                Random r = new Random();

                int iLowValue =  r.Next(0, 500);

                result = GenerateStockTickerData(360, iLowValue, r.Next(iLowValue, iLowValue * 2));

                _cache.UpdateCache(symbol, result);
            }
            else
            {
                result = _cache.GetFromCache(symbol);
            }

            return result;
        }

        /// <summary>
        /// Generates the stock ticker data.
        /// </summary>
        /// <param name="numberOfRecords">The number of records.</param>
        /// <returns></returns>
        private IEnumerable<StockTickerData> GenerateStockTickerData(int numberOfRecords, int lowStockValue, int highStockValue)
        {
            List<StockTickerData> result = new List<StockTickerData>();

            Random r = new Random();

            for (int i = 0; i < numberOfRecords; i++)
            {
                result.Add(new StockTickerData
                {
                    Date = DateTime.Now.AddDays(-1 * (numberOfRecords - i)),
                    Close = r.Next(lowStockValue, highStockValue),
                    Open = r.Next(lowStockValue, highStockValue),
                    Volume = r.Next(0, 1024)
                });
            }

            return result;
        }
    }
}
