﻿using System;
using System.Windows;
using System.Collections.Generic;
using Infragistics.Silverlight.Applications.Stocks.Models;

namespace Infragistics.Silverlight.Applications.Stocks.Services
{
    public class StockDataCacheService : IApplicationService
    {
        #region Private Member Variables
        private Dictionary<string, CacheItem<IEnumerable<StockTickerData>>> _cacheData = new Dictionary<string, CacheItem<IEnumerable<StockTickerData>>>();
        
        private TimeSpan _cacheDuration = new TimeSpan(0, 15, 0);
        #endregion Private Member Variables

        #region IApplicationService
        /// <summary>
        /// Called by an application in order to initialize the application extension service.
        /// </summary>
        /// <param name="context">Provides information about the application state.</param>
        public void StartService(ApplicationServiceContext context)
        {
        }

        /// <summary>
        /// Called by an application in order to stop the application extension service.
        /// </summary>
        public void StopService()
        {
        }
        #endregion IApplicationService

        #region Public Methods
        /// <summary>
        /// Sets the duration of the cache.
        /// </summary>
        /// <param name="duration">The duration.</param>
        public void SetCacheDuration(TimeSpan duration)
        {
            _cacheDuration = duration;

            UpdateCache();
        }

        /// <summary>
        /// Updates the cache.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <param name="data">The data.</param>
        public void UpdateCache(string symbol, IEnumerable<StockTickerData> data)
        {
            if (_cacheData.ContainsKey(symbol))
            {
                _cacheData[symbol].Data = data;
            }
            else
            {
                _cacheData.Add(symbol, new CacheItem<IEnumerable<StockTickerData>> { Data = data });
            }
        }

        /// <summary>
        /// Gets from cache.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <returns></returns>
        public IEnumerable<StockTickerData> GetFromCache(string symbol)
        {
            if (HasCacheExpired(symbol)) return null;

            return _cacheData[symbol].Data;
        }

        /// <summary>
        /// Determines whether [has cache expired] [the specified symbol].
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <returns>
        /// 	<c>true</c> if [has cache expired] [the specified symbol]; otherwise, <c>false</c>.
        /// </returns>
        public bool HasCacheExpired(string symbol)
        {
            UpdateCache();

            if (!IsSymbolInCache(symbol)) return true;

            return _cacheData[symbol].IsExpired;
        }

        /// <summary>
        /// Determines whether [is symbol in cache] [the specified symbol].
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <returns>
        /// 	<c>true</c> if [is symbol in cache] [the specified symbol]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsSymbolInCache(string symbol)
        {
            return _cacheData.ContainsKey(symbol);
        }
        #endregion Public Methods

        #region Private Methods
        /// <summary>
        /// Updates the cache.
        /// </summary>
        private void UpdateCache()
        {
            foreach (var key in _cacheData.Keys)
            {
				if ((_cacheData[key].LastUpdate != DateTime.MinValue) && 
					((_cacheData[key].LastUpdate - DateTime.Now) > _cacheDuration))
                {
                    _cacheData[key].IsExpired = true;
                }
            }
        }
        #endregion Private Methods
    }

    public class CacheItem<T>
    {
        private T _data;

        public T            Data { get { return _data; } set { _data = value; IsExpired = false; LastUpdate = DateTime.Now; } }
        public DateTime     LastUpdate { get; private set; }
        public bool         IsExpired { get; set; }
    }
}
