﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization.Json;
using System.Threading;
using System.Windows;

using Infragistics.Silverlight.Applications.Stocks.Models;

namespace Infragistics.Silverlight.Applications.Stocks.Services
{
    public class YahooStockDataService : IApplicationService, IStockDataProvider
	{
		#region Private Static Variables
		private const string _urlStockData		= "http://pipes.yahooapis.com/pipes/pipe.run?_id=98aa7fee9ad07efbdbde5699c53f1bcb&_render=json&s={0}&a={1}&b={2}&c={3}&d={4}&e={5}&f={6}&g=d";
		private const string _urlStockDetails	= "http://pipes.yahooapis.com/pipes/pipe.run?_id=3e1b7fc9a1a63ea0772d20ce4573d792&_render=json&symbol={0}";
		#endregion Private Static Variables

		#region Private Memeber Variables
        private StockDataCacheService _cache;
		#endregion Private Member Variables

		#region IApplicationService Members
		/// <summary>
		/// Called by an application in order to initialize the application extension service.
		/// </summary>
		/// <param name="context">Provides information about the application state.</param>
		void IApplicationService.StartService(ApplicationServiceContext context)
		{
            _cache = ServiceLocator.GetInstance<StockDataCacheService>();
		}

		/// <summary>
		/// Called by an application in order to stop the application extension service.
		/// </summary>
		void IApplicationService.StopService()
		{
		}
		#endregion IApplicationService Members

		#region Public Methods
		/// <summary>
		/// Refreshes the details.
		/// </summary>
		/// <param name="symbols">The symbols.</param>
		/// <param name="callback">The callback.</param>
        public void RefreshDetails(IList<string> symbols, Action<IDictionary<string, StockTickerDetails>, Exception, bool> callback)
		{
			WebClient wc = new WebClient();

			wc.OpenReadCompleted += OnRefreshDetailsComplete;
			wc.OpenReadAsync(new Uri(string.Format(_urlStockDetails,
										symbols.Aggregate((x, y) => string.Format("{0}+{1}", x, y)))), 
										callback);
		}

		/// <summary>
		/// Refreshes the data.
		/// </summary>
		/// <param name="symbol">The symbol.</param>
		/// <param name="startDate">The start date.</param>
		/// <param name="callback">The callback.</param>
        public void RefreshData(string symbol, DateTime startDate, Action<string, IEnumerable<StockTickerData>, Exception, bool> callback)
		{
			RefreshData(symbol, startDate, DateTime.Now.AddDays(-1.0), callback);
		}

		/// <summary>
		/// Refreshes the data.
		/// </summary>
		/// <param name="symbol">The symbol.</param>
		/// <param name="startDate">The start date.</param>
		/// <param name="endDate">The end date.</param>
		/// <param name="callback">The callback.</param>
        public void RefreshData(string symbol, DateTime startDate, DateTime endDate, Action<string, IEnumerable<StockTickerData>, Exception, bool> callback)
		{
            // Is this a redundant query?
            if (_cache.IsSymbolInCache(symbol) && !_cache.HasCacheExpired(symbol))
            {
                SynchronizationContext.Current.Post(x => callback(symbol, (IEnumerable<StockTickerData>)x, null, true), _cache.GetFromCache(symbol));

                return;
            }

			WebClient wc = new WebClient();

			wc.OpenReadCompleted += OnRefreshDataComplete;
			wc.OpenReadAsync(new Uri(string.Format(_urlStockData,
														symbol,
														startDate.Month - 1,
														startDate.Day,
														startDate.Year,
														endDate.Month - 1,
														endDate.Day,
														endDate.Year)),
                                                        new Tuple<string, Action<string, IEnumerable<StockTickerData>, Exception, bool>>(symbol, callback));

		}
		#endregion Public Methods

		#region Private Methods
		/// <summary>
		/// Called when [refresh details complete].
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.Net.OpenReadCompletedEventArgs"/> instance containing the event data.</param>
		private void OnRefreshDetailsComplete(object sender, OpenReadCompletedEventArgs e)
		{
            var callback = (Action<IDictionary<string, StockTickerDetails>, Exception, bool>)e.UserState;

            if (e.Error != null)
			{
				//TODO: We should add logging or throw exception here
                
                SynchronizationContext.Current.Post(x => callback(null, e.Error, false), null);
				return;
			}

			var result = ProcessStockDetails(e.Result);

			

			SynchronizationContext.Current.Post(x => callback((IDictionary<string, StockTickerDetails>)x, null, false), result);
		}

		/// <summary>
		/// Called when [refresh data complete].
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.Net.OpenReadCompletedEventArgs"/> instance containing the event data.</param>
		private void OnRefreshDataComplete(object sender, OpenReadCompletedEventArgs e)
		{
            var objectState = (Tuple<string, Action<string, IEnumerable<StockTickerData>, Exception, bool>>)e.UserState;

            var symbol = objectState.Item1;
            var callback = objectState.Item2;
            
            if (e.Error != null)
			{
				//TODO: We should add logging or throw exception here

                SynchronizationContext.Current.Post(x => callback(null, null, e.Error, false), null);
			    return;
			}

			var result = ProcessStockData(e.Result);
            _cache.UpdateCache(symbol, result);

			SynchronizationContext.Current.Post(x => callback(symbol, (IEnumerable<StockTickerData>)x, null, false), result);
		}

		/// <summary>
		/// Processes the stock details.
		/// </summary>
		/// <param name="data">The data.</param>
		/// <returns></returns>
		private IDictionary<string, StockTickerDetails> ProcessStockDetails(Stream data)
		{
#if DEBUG
			string jsonData;
			StreamReader sr = new StreamReader(data);
			jsonData = sr.ReadToEnd();
			data.Position = 0;
#endif
			try
			{
				DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(JsonRoot<StockTickerDetails>),
																					   new[]
		                                                                       			{
		                                                                       				typeof (JsonRoot<StockTickerDetails>),
		                                                                       				typeof (JsonContent<StockTickerDetails>),
		                                                                       				typeof (StockTickerDetails),
		                                                                       				typeof (List<StockTickerDetails>)
		                                                                       			});

				var processedData = (JsonRoot<StockTickerDetails>)(serializer.ReadObject(data));

				var result = new Dictionary<string, StockTickerDetails>(processedData.value.items.Count);

				foreach (var item in processedData.value.items)
				{
					result.Add(item.Symbol, item);
				}

				return result;
			}
			catch (System.Exception ex)
			{
#if DEBUG
				throw new Exception("Invalid JASON: " + jsonData, ex);
#else
				throw new Exception("Invalid JASON", ex);
#endif
			}
		}

		/// <summary>
		/// Processes the stock data.
		/// </summary>
		/// <param name="data">The data.</param>
		/// <returns></returns>
		private IEnumerable<StockTickerData> ProcessStockData(Stream data)
		{
#if DEBUG
			string jsonData;
			StreamReader sr = new StreamReader(data);
			jsonData = sr.ReadToEnd();
			data.Position = 0;
#endif
			try
			{
				DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(JsonRoot<StockTickerData>),
																					   new[]
			                                                                       		{
			                                                                       			typeof (JsonRoot<StockTickerData>),
			                                                                       			typeof (JsonContent<StockTickerData>),
			                                                                       			typeof (StockTickerData),
			                                                                       			typeof (List<StockTickerData>)
			                                                                       		});
				var result = (JsonRoot<StockTickerData>)(serializer.ReadObject(data));

				return result.value.items.OrderBy(x => x.Date);
			}
			catch (System.Exception ex)
			{
#if DEBUG
				throw new Exception("Invalid JASON: " + jsonData, ex);
#else
				throw new Exception("Invalid JASON", ex);
#endif
			}
		}
		#endregion Private Methods
	}
}
