﻿using System;
using System.Windows;
using System.ComponentModel;
using Infragistics.Silverlight.Applications.Stocks.Models;
using System.Collections.Generic;
using System.Windows.Input;
using System.Linq;
using System.Windows.Media;

namespace Infragistics.Silverlight.Applications.Stocks.ViewModels
{
	/// <summary>
	/// 
	/// </summary>
	public class StockInfoViewModel : INotifyPropertyChanged
	{
		#region Private Member Variables
		private static Dictionary<int, int> 	_rangeIntervalLookup;

		private StockTickerDetails				_details = new StockTickerDetails();
	    private bool                            _isDetailsLoaded;
		private IEnumerable<StockTickerData>	_data;
		private StockViewModel					_parent;

        private int 	_rangeFilter	= 0;
		private int 	_rangeInterval	= 4;
		private Brush	_brush			= new SolidColorBrush(Colors.Yellow);
		#endregion Private Member Variables

		#region Constructor
		/// <summary>
		/// Initializes the <see cref="StockInfoViewModel"/> class.
		/// </summary>
		static StockInfoViewModel()
		{
			_rangeIntervalLookup = new Dictionary<int, int>();

			_rangeIntervalLookup[120] 	= 60;	// 10 Years
			_rangeIntervalLookup[60] 	= 60;	// 5 Years
			_rangeIntervalLookup[48] 	= 60;	// 4 Years
			_rangeIntervalLookup[36] 	= 60;	// 3 Years
			_rangeIntervalLookup[24] 	= 60;	// 2 Years
			_rangeIntervalLookup[12] 	= 60;	// 1 Year
			_rangeIntervalLookup[6] 	= 30;	// 6 Months
			_rangeIntervalLookup[3] 	= 21;	// 3 Months
			_rangeIntervalLookup[2] 	= 14;	// 2 Months
			_rangeIntervalLookup[1] 	= 7;	// 1 Month
			_rangeIntervalLookup[0]		= 120; 	// unlimited
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="StockInfoViewModel"/> class.
		/// </summary>
		/// <param name="parent">The parent.</param>
		/// <param name="symbol">The symbol.</param>
		public StockInfoViewModel(StockViewModel parent, string symbol, Brush brush)
		{
            Brush = brush;

			_parent			= parent;
			_details.Symbol = symbol;

            RangeFilter = 1;
		}
		#endregion Constructor

		#region Public Properties
		/// <summary>
		/// Gets the symbol.
		/// </summary>
		/// <value>The symbol.</value>
		public string Symbol
		{
			get { return Details != null ? Details.Symbol : string.Empty; }
		}

		/// <summary>
		/// Gets or sets the details.
		/// </summary>
		/// <value>The details.</value>
		public StockTickerDetails Details
		{
			get { return _details; }
            set { if (value == _details) return; _details = value; IsDetailsLoaded = true; RaisePropertyChanged("Details"); RaisePropertyChanged("Symbol"); }
		}

	    public bool IsDetailsLoaded
	    {
            get { return _isDetailsLoaded; }
            set { _isDetailsLoaded = value; RaisePropertyChanged("IsDetailsLoaded"); }
	    }

		/// <summary>
		/// Gets or sets the data.
		/// </summary>
		/// <value>The data.</value>
		public IEnumerable<StockTickerData> Data
		{
			get { return _data; }
			set { if (value == _data) return; _data = value; RaisePropertyChanged("Data"); RaisePropertyChanged("FilteredData"); Parent.RaisePropertyChanged("Tickers"); Parent.RaisePropertyChanged("SelectedStock"); }
		}

        /// <summary>
        /// Returns Stock Data based on the current Range Filter (if range filter = 0 than all data is returned)
        /// </summary>
		public IEnumerable<StockTickerData> FilteredData
        {
            get
            {
                if (_data == null) return _data;
                if (_rangeFilter == 0) return _data;

                DateTime dt = DateTime.Now.AddMonths(-1 * RangeFilter);

                return _data.Where(x => x.Date > dt);
            }
        }

        /// <summary>
        /// Sets a filter for the data (based in number of months)
        /// </summary>
        public int RangeFilter
        {
            get { return _rangeFilter; }
            set 
			{
				if (value == _rangeFilter) return;

				// This indicates a number in months
				_rangeFilter 	= value;

				// What is the X Axises interval going to look like?
				bool bFoundInterval = false;
				int i = _rangeFilter;

				// Set the default value (the unlimited value)
				_rangeInterval = _rangeIntervalLookup[0];

				// Find the one we are looking for or the next closest
				while (!bFoundInterval && i > 0)
				{
					if (_rangeIntervalLookup.ContainsKey(i))
					{
						_rangeInterval = _rangeIntervalLookup[i];
						bFoundInterval = true;
					} else i--;
				}
			
				RaisePropertyChanged("RangeFilter"); 
				RaisePropertyChanged("FilteredData");
				RaisePropertyChanged("RangeInterval");
			}

        }

		/// <summary>
		/// Gets or sets the range interval.
		/// </summary>
		/// <value>The range interval.</value>
		public int RangeInterval
		{
			get { return _rangeInterval; }
			set { if (value == _rangeInterval) return; _rangeInterval = value; RaisePropertyChanged("RangeInterval"); RaisePropertyChanged("FilteredData"); }
		}

		/// <summary>
		/// Gets the parent.
		/// </summary>
		/// <value>The parent.</value>
		public StockViewModel Parent
		{
			get { return _parent; }
		}

        /// <summary>
        /// Gets or sets the brush for this stock.
        /// </summary>
        /// <value>The brush.</value>
        public Brush Brush
        {
            get;
            private set;
        }
		#endregion Public Properties

		#region Public Commands
		#endregion Public Commands

		#region Public Methods
		#endregion Public Methods

		#region INotifyPropertyChanged Members
		public event PropertyChangedEventHandler PropertyChanged;
		/// <summary>
		/// Raises the property changed.
		/// </summary>
		/// <param name="propertyName">Name of the property.</param>
		private void RaisePropertyChanged(string propertyName)
		{
			if (PropertyChanged == null) return;

			PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}
		#endregion
	}
}
