﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Collections.ObjectModel;
using System.Windows.Input;
using System.Windows.Threading;
using Infragistics.Silverlight.Applications.Stocks.Services;
using Infragistics.Silverlight.Applications.Stocks.Models;
using Infragistics.Silverlight.Commands;
using System.Windows;
using System.Windows.Media;
using System.Xml.Linq;
using System.Threading;
using System.IO.IsolatedStorage;
using System.IO;
using Microsoft.Phone.Net.NetworkInformation;

namespace Infragistics.Silverlight.Applications.Stocks.ViewModels
{
	/// <summary>
	/// 
	/// </summary>
	public class StockViewModel : INotifyPropertyChanged
	{
		#region Private Member Variables
        private IStockDataProvider _stockService;

		private ObservableCollection<StockInfoViewModel> _stocks = new ObservableCollection<StockInfoViewModel>();
		
		private StockInfoViewModel _selectedStock = null;

		private int		    _currentPeriod	= 1;

        private ICommand    _commandAddStock;
        private ICommand    _commandRemoveStock;
        private ICommand    _commandSelectStock;
        private ICommand    _commandStockDisplayRange;
		private ICommand 	_commandMoveToNextStock;

		private DispatcherTimer _timer;

		private int					_retrieveDataCount;

        private BrushCollection 	_stockColors;

	    private bool _detailsUpdated = false;
	    private bool _hasConnection = true;
	    private object _connectionLock = new object();

		private List<string>                            _symbolsToAdd = new List<string>();
        private string 									_stockSearchFilter;
		private static IList<StockSearchItemViewModel> 	_stockSearchList;
        private static IList<StockSearchItemViewModel> _stockTopList;
        #endregion Private Member Variables

		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="StockViewModel"/> class.
		/// </summary>
		public StockViewModel()
		{
			SynchronizationContext.Current.Post(x => OnLoadStockSearchData(), null);
			
			_stockService = ServiceLocator.GetInstance<YahooStockDataService>();

            RefreshStockDetailsTick(null, null);

			_timer = new DispatcherTimer();
			_timer.Interval = new System.TimeSpan(0, 0, 10);
			_timer.Tick += RefreshStockDetailsTick;
			_timer.Start();

			RefreshData();
		}
		#endregion Constructors

		#region Public Properties
		/// <summary>
		/// Gets the tickers.
		/// </summary>
		/// <value>The tickers.</value>
		public ObservableCollection<StockInfoViewModel> Tickers
		{
			get { return _stocks; }
		}

		/// <summary>
		/// Gets or sets the selected stock.
		/// </summary>
		/// <value>The selected stock.</value>
		public StockInfoViewModel SelectedStock
		{
			get { return _selectedStock; }
			set { if (value == _selectedStock) return; _selectedStock = value == null && _stocks.Count > 0 ? _stocks.First() : value; RaisePropertyChanged("SelectedStock"); }
		}

		/// <summary>
		/// Gets the stocks.
		/// </summary>
		/// <value>The stocks.</value>
		public IEnumerable<StockSearchItemViewModel> StockSearchList
		{
			get
			{
				if (string.IsNullOrEmpty(_stockSearchFilter)) return _stockTopList;

				var filterLower = _stockSearchFilter.ToLower();

                //return _stockSearchList.Where(x => { var symbolLower = x.Symbol.ToLower(); return symbolLower.StartsWith(filterLower); }).Take(10);
                var res = new List<StockSearchItemViewModel>(10);
			    foreach (var stockSearchItemViewModel in _stockSearchList)
			    {
                    var symbolLower = stockSearchItemViewModel.Symbol.ToLower();
                    if (symbolLower.StartsWith(filterLower))
                    {
                        res.Add(stockSearchItemViewModel);
                        if (res.Count == 10)
                        {
                            return res;
                        }
                    }
			    }

			    return res;
			}
		}

		/// <summary>
		/// Gets or sets the stock search filter.
		/// </summary>
		/// <value>The stock search filter.</value>
		public string StockSearchFilter
		{
			get { return _stockSearchFilter; }
			set { _stockSearchFilter = value; RaisePropertyChanged("StockSearchFilter"); RaisePropertyChanged("StockSearchList"); }
		}

		/// <summary>
		/// Gets or sets a value indicating whether this instance is retrieving data.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance is retrieving data; otherwise, <c>false</c>.
		/// </value>
		public bool IsInitialDataLoad { get { return _retrieveDataCount == 1; } }
		#endregion Public Properties

		#region Public Commands
		/// <summary>
		/// Gets the add stock command.
		/// </summary>
		/// <value>The add stock command.</value>
		public ICommand AddStockCommand
		{
			get
			{
				if (_commandAddStock == null)
				{
					_commandAddStock = new RelayCommand<string>(
					  p => { EnqueueAddSymbolToWatchList(p); },
					  p => { return true; });

					RaisePropertyChanged("AddStockCommand");
				}
				return (_commandAddStock);
			}
		}

		/// <summary>
		/// Gets the remove stock command.
		/// </summary>
		/// <value>The remove stock command.</value>
		public ICommand RemoveStockCommand
		{
			get
			{
				if (_commandRemoveStock == null)
				{
					_commandRemoveStock = new RelayCommand<string>(
					  p => { RemoveSymbolFromWatchList(p); },
					  p => { return true; }
					  );

					RaisePropertyChanged("RemoveStockCommand");
				}
				return (_commandRemoveStock);
			}
		}

		/// <summary>
		/// Gets the select stock command.
		/// </summary>
		/// <value>The select stock command.</value>
		public ICommand SelectStockCommand
		{
			get
			{
				if (_commandSelectStock == null)
				{
					_commandSelectStock = new RelayCommand<string>(
						p => { SetSelectedStock(p); },
						p => { return true; }
						);

					RaisePropertyChanged("SelectStockCommand");
				}

				return (_commandSelectStock);
			}
		}

		/// <summary>
		/// Gets the move to next stock.
		/// </summary>
		/// <value>The move to next stock.</value>
		public ICommand MoveToNextStock
		{
			get
			{
				if (_commandMoveToNextStock == null)
				{
					_commandMoveToNextStock = new RelayCommand<string>(
						p => { MoveToStock(int.Parse(p)); },
						p => { return true; }
						);
				}

				return (_commandMoveToNextStock);
			}
		}

        /// <summary>
        /// Gets the stock display range command
        /// </summary>
        public ICommand StockDisplayRangeCommand
        {
            get
            {
                if (_commandStockDisplayRange == null)
                {
                    _commandStockDisplayRange = new RelayCommand<string>(
                        p => { SetStockDisplayRange(int.Parse(p.ToString())); },
                        p => { return true; }
                    );

                    RaisePropertyChanged("StockDisplayRangeCommand");
                }

                return (_commandStockDisplayRange);
            }
        }
		#endregion Public Commands

        #region Public Properties
        public static DependencyProperty StockColorsProperty = DependencyProperty.Register("StockColors", 
                                                                                            typeof(BrushCollection), 
                                                                                            typeof(StockViewModel), 
                                                                                            new PropertyMetadata(null, OnStockColorsChanged));

        /// <summary>
        /// Gets or sets the stock colors.
        /// </summary>
        /// <value>The stock colors.</value>
        public BrushCollection StockColors
        {
            get { return _stockColors; }
            set { if (value == _stockColors) return; _stockColors = value; RaisePropertyChanged("StockColors"); }
        }

        /// <summary>
        /// Called when [stock colors changed].
        /// </summary>
        /// <param name="d">The d.</param>
        /// <param name="e">The <see cref="System.Windows.DependencyPropertyChangedEventArgs"/> instance containing the event data.</param>
        private static void OnStockColorsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            d.SetValue(StockColorsProperty, e.NewValue);
        }

		/// <summary>
		/// Gets a value indicating whether the selected stock instances details have been populated.
		/// </summary>
		/// <value>
		/// 	<c>true</c> if this instance is details populated; otherwise, <c>false</c>.
		/// </value>
		public bool IsDetailsPopulated { get { return _detailsUpdated && SelectedStock != null && SelectedStock.Details != null && !string.IsNullOrEmpty(SelectedStock.Details.Symbol);} }

        public bool HasConnection { get { return _hasConnection; } set { _hasConnection = value; RaisePropertyChanged("HasConnection"); } }

        #endregion Public Properties

        #region Public Methods

        /// <summary>
		/// Adds the tick to watch list.
		/// </summary>
		/// <param name="symbol">The symbol.</param>
		public void AddSymbolToWatchList(string symbol)
		{
			var sivm = GetStockInfoBySymbolName(symbol);

			if (sivm != null) return;

            int iBrushIndex = _stocks.Count();

			// If we have more stocks then brushes, let find the next brush to use
			Brush b = null;
			if (_stockColors != null)
			{
				while (iBrushIndex > _stockColors.Count - 1)
				{
					iBrushIndex = iBrushIndex - _stockColors.Count;
				}

				b = iBrushIndex < _stockColors.Count ? _stockColors[iBrushIndex] : null;
			}

            var stock = new StockInfoViewModel(this, symbol, b);

            if (_stocks.Count > 0)
            {
                stock.RangeFilter = _stocks.Last().RangeFilter;
            }
            
            _stocks.Add(stock);
            
			RaisePropertyChanged("Tickers");

            SelectedStock = _stocks.Last();
		}

        /// <summary>
        /// Adds the tick to watch list.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        public void EnqueueAddSymbolToWatchList(string symbol)
        {
            var timer = new DispatcherTimer();
            timer.Interval = new System.TimeSpan(0, 0, 0, 0, 300);
            timer.Tick += AddSymbolTick;
            lock (_symbolsToAdd)
            {
                _symbolsToAdd.Add(symbol);
            }
            timer.Start();
        }

	    private void AddSymbolTick(object sender, EventArgs e)
	    {
	        ((DispatcherTimer)sender).Stop();
	        string symbol = null;
            lock (_symbolsToAdd)
            {
                if (_symbolsToAdd.Count > 0)
                {
                    symbol = _symbolsToAdd.First();
                    _symbolsToAdd.Remove(symbol);
                }
            }
	        
            if (symbol != null)
            {
                AddSymbolToWatchList(symbol);
            }
	    }

	    /// <summary>
		/// Removes the ticker from watch list.
		/// </summary>
		/// <param name="symbol">The symbol.</param>
		public void RemoveSymbolFromWatchList(string symbol)
		{
			var sivm = GetStockInfoBySymbolName(symbol);

			if (sivm == null) return;

			_stocks.Remove(sivm);

            // If we just removed the active stock, we should reset it.
            if (_stocks.Count > 0 && (sivm == SelectedStock || SelectedStock == null))
            {
                SelectedStock = _stocks.First();
            }

			RaisePropertyChanged("Tickers");
            RaisePropertyChanged("SelectedStock");
		}

		/// <summary>
		/// Moves to stock based on index.
		/// </summary>
		/// <param name="index">The index.</param>
		public void MoveToStock(int index)
		{
			int iCurrentIndex = _stocks.IndexOf(SelectedStock);

			iCurrentIndex += index;

			if (iCurrentIndex < 0 || iCurrentIndex >= _stocks.Count) return;

			SelectedStock = _stocks[iCurrentIndex];
		}

		/// <summary>
		/// Sets the selected stock.
		/// </summary>
		/// <param name="symbol">The symbol.</param>
		public void SetSelectedStock(string symbol)
		{
			var sivm = GetStockInfoBySymbolName(symbol);

			if (sivm == null) return;

			SelectedStock = sivm;
		}

        /// <summary>
        /// Set the Range Filter for the currently selected stock
        /// </summary>
        /// <param name="numberOfMonths"></param>
        public void SetStockDisplayRange(int numberOfMonths)
        {
            /*if (SelectedStock == null) return;

            SelectedStock.RangeFilter = numberOfMonths;*/
            
            foreach (var stockInfoViewModel in Tickers)
            {
                stockInfoViewModel.RangeFilter = numberOfMonths;
            }

            
        }

		/// <summary>
		/// Refreshes the details.
		/// </summary>
		public void RefreshDetails()
		{
			try
			{
				_stockService.RefreshDetails(_stocks.Select(x => x.Symbol).ToList(), OnRefreshDetails);

				RefreshData();
			}
			catch {}
		}

		/// <summary>
		/// Refreshes the data.
		/// </summary>
		public void RefreshData()
		{
			try
			{
				_retrieveDataCount++;
				RaisePropertyChanged("IsInitialDataLoad");

				foreach (var item in _stocks)
				{
					_stockService.RefreshData(item.Symbol, DateTime.Now.AddYears(-1 * _currentPeriod), OnRefreshData);
				}
			}
			catch { }
		}

		/// <summary>
		/// Saves the current list of stocks to local isolated storage.
		/// </summary>
		public void Save()
		{
			// Do we have anything to save?
			//if (_stocks.Count == 0) return;

			//string stocks = _stocks.Select(x => x.Symbol).Aggregate((x, y) => string.Format("{0};{1}"));
		    string stocks = "";
		    foreach (StockInfoViewModel stockInfoViewModel in _stocks)
		    {
		        stocks += stockInfoViewModel.Symbol + ";";
		    }
            if (SelectedStock != null)
            {
                stocks += SelectedStock.Symbol;
            }

			SaveData(stocks, "InfragisticsMobileStocks.data");
		}

		/// <summary>
		/// Saves a list of stocks from local isolated storage.
		/// </summary>
		public void Load()
		{
		    string[] stocks = LoadData("InfragisticsMobileStocks.data").Split(';');

		    for (int i = 0; i < stocks.Length - 1; i++)
		    {
		        string stock = stocks[i];
		        if (string.IsNullOrEmpty(stock)) continue;

		        AddSymbolToWatchList(stock);
		    }
            if (stocks.Length > 0)
            {
                SetSelectedStock(stocks.Last());
            }
		}

	    #endregion Public Methods

		#region Private Methods
        /// <summary>
        /// Gets the name of the stock info by symbol.
        /// </summary>
        /// <param name="symbol">The symbol.</param>
        /// <returns></returns>
		StockInfoViewModel GetStockInfoBySymbolName(string symbol)
		{
			var sivm = (from item in _stocks
						where item.Symbol == symbol
						select item).DefaultIfEmpty(null).FirstOrDefault();

			return sivm;
		}

		/// <summary>
		/// Refreshes the stock details tick.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		void RefreshStockDetailsTick(object sender, EventArgs e)
		{
            RefreshDetails();
		}

		/// <summary>
		/// Called when [refresh details].
		/// </summary>
		/// <param name="data">The data.</param>
        private void OnRefreshDetails(IDictionary<string, StockTickerDetails> data, Exception e, bool fromCache)
		{
            if (e != null)
            {
                UpdateHasConnectionSafely(false);
            }
            else
            {
                foreach (var sivm in _stocks)
                {
                    // It is possible a new stock was added while a query was in progress
                    // If so, we can ignore it as it will be picked up on the next query
                    if (data.ContainsKey(sivm.Symbol))
                    {
                        sivm.Details = data[sivm.Symbol];
                    }
                }

                _detailsUpdated = true;

                RaisePropertyChanged("SelectedStock");
                RaisePropertyChanged("Tickers");
                RaisePropertyChanged("IsDetailsPopulated");
            }
		}

		/// <summary>
		/// Called when [refresh data].
		/// </summary>
		/// <param name="symbol">The symbol.</param>
		/// <param name="data">The data.</param>
        private void OnRefreshData(string symbol, IEnumerable<StockTickerData> data, Exception e, bool fromCache)
		{
            if (e != null)
            {
                UpdateHasConnectionSafely(false);
            }
            else
            {
                if (!fromCache || NetworkInterface.NetworkInterfaceType != NetworkInterfaceType.None)
                {
                    UpdateHasConnectionSafely(true);
                }
                else
                {
                    UpdateHasConnectionSafely(false);
                }

                _retrieveDataCount++;
                RaisePropertyChanged("IsInitialDataLoad");

                var sivm = GetStockInfoBySymbolName(symbol);

                if (sivm == null)
                {
                    return;
                }

                sivm.Data = data;

                RaisePropertyChanged("SelectedStock");
                RaisePropertyChanged("Tickers");
            }
		}

        private void UpdateHasConnectionSafely(bool value)
        {
            lock (_connectionLock)
            {
                if (HasConnection != value)
                {
                    HasConnection = value;
                }
            }
        }

		/// <summary>
		/// Called when [load stock search data].
		/// </summary>
		private void OnLoadStockSearchData()
		{
			try
			{
				XDocument xDoc = XDocument.Load("Assets/StockSymbols.xml");

				_stockSearchList = (from xNode in xDoc.Element("stockSymbols").Elements("stock")
									select new StockSearchItemViewModel
									{
										Symbol = xNode.Attribute("Symbol").Value,
										Exchange = xNode.Attribute("Source").Value,
										Name = xNode.Attribute("Description").Value,
										Parent = this
									}).ToList();

                xDoc = XDocument.Load("Assets/TopStockSymbols.xml");

                _stockTopList = (from xNode in xDoc.Element("stockSymbols").Elements("stock")
                                    select new StockSearchItemViewModel
                                    {
                                        Symbol = xNode.Attribute("Symbol").Value,
                                        Exchange = xNode.Attribute("Source").Value,
                                        Name = xNode.Attribute("Description").Value,
                                        Parent = this
                                    }).ToList();

                RaisePropertyChanged("StockSearchList");
			}
			catch { }
		}

		/// <summary>
		/// Saves the data.
		/// </summary>
		/// <param name="data">The data.</param>
		/// <param name="fileName">Name of the file.</param>
		private void SaveData(string data, string fileName)
		{
			using (IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForApplication())
			{
				using (IsolatedStorageFileStream isfs = new IsolatedStorageFileStream(fileName, FileMode.Create, isf))
				{
					using (StreamWriter sw = new StreamWriter(isfs))
					{
						sw.Write(data); 
						
						sw.Close();
					}
				}
			}
		}

		/// <summary>
		/// Loads the data.
		/// </summary>
		/// <param name="fileName">Name of the file.</param>
		/// <returns></returns>
		private string LoadData(string fileName)
		{
			using (IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForApplication())
			{
				if (!isf.FileExists(fileName)) return string.Empty;

				using (IsolatedStorageFileStream isfs = new IsolatedStorageFileStream(fileName, FileMode.Open, isf))
				{
					using (StreamReader sr = new StreamReader(isfs))
					{
						return sr.ReadToEnd();
					}
				}
			}
		}
		#endregion Private Methods

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;
		/// <summary>
		/// Raises the property changed.
		/// </summary>
		/// <param name="propertyName">Name of the property.</param>
		internal void RaisePropertyChanged(string propertyName)
		{
			if (PropertyChanged == null) return;
			
			PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}
		#endregion
	}
}
